@@ -16,6 +16,7 @@ from cnvrg.helpers.libs_helper import check_if_lib_exists
 from enum import Enum
 from typing import List
 import numpy as np
+import types
 import os
 import time
 
@@ -33,6 +34,7 @@ class Experiment(CnvrgJob):
             raise UserError("Cant create an experiment without slug and outside experiment run")
         slug = slug or CURRENT_JOB_ID
         super(Experiment, self).__init__(slug, env_helper.EXPERIMENT, Project(url_join(owner, project_slug)))
+        self.title = None
         self.__data = self.__get_experiment()
 
     @staticmethod
@@ -46,33 +48,76 @@ class Experiment(CnvrgJob):
         return exp
 
     @staticmethod
-    def __run_callable(callable, title=None, project=None, cwd=None, library=None, **kwargs):
+    def __run_callable(function_command, title=None, project=None, cwd=None, library=None, **kwargs):
         experiment = Experiment.init(title=title, project=project, library=library, **kwargs)
-        exit_status = control_stdout_helper.run_callable(callable, callback=lambda x: experiment.log(x), err_callback=lambda x: experiment.log(x, log_type=LOGS_TYPE_ERROR))
+        exit_status = control_stdout_helper.run_callable(function_command, arguments=[experiment], callback=lambda x: experiment.log(x), err_callback=lambda x: experiment.log(x, log_type=LOGS_TYPE_ERROR))
         experiment.finish(exit_status=exit_status)
         return experiment
 
     @staticmethod
     def __run_local(cmd, **kwargs):
-        callable = lambda: spawn_helper.run_sync(cmd, print_output=True, cwd=kwargs["cwd"])
-        return Experiment.__run_callable(callable, cmd=cmd, **kwargs)
+        function_command = lambda *args: spawn_helper.run_sync(cmd, print_output=True, cwd=kwargs["cwd"])
+        return Experiment.__run_callable(function_command, cmd=cmd, **kwargs)
+
 
     @staticmethod
-    def run(command=None, callable=None, title=None, project=None, compute=None, computes=None, datasets=None, dataset=None, local=False, library=None, working_directory=None):
+    def sync(**kwargs):
+        Project().sync(**{**CnvrgJob.current_job_sync_args(), **kwargs})
+
+
+    def restart(self, message=None, sync=True):
+        if sync: Experiment.sync()
+        if not in_experiment():
+            raise UserError("Cant restart an experiment outside the experiment itself.")
+        return apis_helper.post(url_join(self._base_url(), 'restart'), data={"message": message})
+
+    @staticmethod
+    def get_utilization():
+        return spawn_helper.analyze_pid(pid=os.getpid())
+
+    @staticmethod
+    def run(command=None, title=None, project=None, compute=None, computes=None, datasets=None, dataset=None, local=False, library=None, working_directory=None, sync_before=True, sync_after=True, **kwargs):
+        function_command = None
         project = project or Project()
+
+        ## sync before
+        if sync_before: project.sync(**CnvrgJob.current_job_sync_args())
+
         ## support
-        computes = computes or wrap_string_to_list(computes) or wrap_string_to_list(compute)
+        computes = computes or wrap_string_to_list(computes) or wrap_string_to_list(compute) or []
+        ### by default (if computes empty or no compute) set the local to True
+        if local: pass
+        elif "local" in computes: local = True
+        elif len(computes) == 0: local = True
+
+        ### if command is function, run it as a function and not as a os command
+        if isinstance(command, types.FunctionType): function_command = command
+
+        ## merge datasets
         datasets = datasets or wrap_string_to_list(datasets) or wrap_string_to_list(dataset)
 
-        if callable:
-            cmd = "func: {func_name}".format(func_name=callable.__name__)
-            return Experiment.__run_callable(callable, title=title, project=project, library=library, cmd=cmd)
+
+
+
+        if function_command:
+            ## set the title to "func: {name of the function}"
+            cmd = "func: {func_name}".format(func_name=function_command.__name__)
+            experiment = Experiment.__run_callable(function_command, title=title, project=project, library=library, cmd=cmd)
+            if not experiment: raise UserError("Cant run experiment, please check your input params")
         elif local:
-            return Experiment.__run_local(cmd=command, title=title, project=project, cwd=working_directory, library=library)
+            ## if its a string command && local=True, run it as a local task (subprocess.Popen)
+            experiment = Experiment.__run_local(cmd=command, title=title, project=project, cwd=working_directory, library=library)
+            if not experiment: raise UserError("Cant run experiment, please check your input params")
         else:
-            e = project.run_task(command, title=title, templates=computes, datasets=datasets, library=library)
+
+            ## perform an api call to create it.
+            e = project.run_task(command, title=title, templates=computes, datasets=datasets, library=library, **kwargs)
             return Experiment(e.get("experiments")[0])
 
+        if sync_after: project.sync(**CnvrgJob.current_job_sync_args(job=experiment['slug']))
+        return experiment
+
+
 
     def log_param(self, key, value=None):
         tag_data = {
@@ -122,6 +167,19 @@ class Experiment(CnvrgJob):
             [callback(l) if l else None for l in job_logs]
 
 
+    @property
+    def title(self):
+        return self["title"]
+
+    @title.setter
+    def title(self, new_title):
+        apis_helper.put(url_join(self._base_url(), 'title'), data={"title": new_title})
+        self.__data = self.__get_experiment()
+
+    def set_title(self,new_title):
+        self.title = new_title
+
+
     def show_chart(self, key, **kwargs):
         """
 
